<?php
/* An RSS Feed Class for Airtime 2.5.x by John Chewter, info@coastal-computing.com */

class rss_feed {	 
	/**
	* Constructor
	*
	* @param array $airtime_db database settings
	* @param string $xmlns XML namespace
	*/
	public function __construct($db_connect, $xmlns, $myhours) {
		// initialize
		$this->xmlns = ($xmlns ? ' ' . $xmlns : '');
		$this->db_connect = $db_connect;
		$fil = pg_connect($this->db_connect);
		
		$sql = 'SELECT keyname, value, type FROM cc_stream_setting';
		$stream_settings = pg_query($sql);

		while($stream_setting = pg_fetch_array($stream_settings,NULL,PGSQL_ASSOC)){
			switch ($stream_setting['keyname']) {
				case 's1_url':
					$link = $stream_setting['value'];
					 break;
				case 's1_description':
					$description = htmlentities($stream_setting['value']);
					break;
				case 's1_name':
					$title = htmlentities($stream_setting['value']);
					break;
			}
		}

		// Init $language with a fallback value
		$language = 'en';
		$timezone = 'TZ';
		
		$sql = 'SELECT keystr, valstr FROM cc_pref';
		$stream_settings = pg_query($sql);
		
		while($stream_setting = pg_fetch_array($stream_settings,NULL,PGSQL_ASSOC)){
			switch ($stream_setting['keystr']) {
				case 'locale':
					// Make the language generic
					$language = str_replace('_', '-', strtolower($stream_setting['valstr']));
					if (strpos($language, '-') !== false) {
						$langs = explode("-", $language);
						$language = $langs[0];
					}
					 break;
				case 'timezone':
					$timezone = $stream_setting['valstr'];
					break;
			}
		}
		
		$ChannelImagePath = (@$_SERVER["HTTPS"] == "on") ? "https://" : "http://";
    	$ChannelImagePath .=$_SERVER["SERVER_NAME"]. dirname($_SERVER["PHP_SELF"]); 
		// Configure the RSS Channel settings.	
		$airtime_rss_channel = array(
		  	"title" => $title,
		  	"link" => $link,
			"image_title" => $title,
		  	"image_link" => $link,	
		  	"description" => $description,
		  	"language" => $language,
		  	"image_url" => $ChannelImagePath."/rss-logo.png",
			"atom_path"  => $ChannelImagePath."/feed.xml",
		);
		
		$this->channel_properties = $airtime_rss_channel;
		$this->site_url = $link;
		$this->site_name = $title;
		$this->full_feed = false;
		$this->timezone = $timezone;
		$this->hours = $myhours;
		pg_close($fil);
	 }
	 
	/*
		* Generate RSS 2.0 feed
		* @return string RSS 2.0 / Atom xml
	*/
	public function create_feed() {
		// Header
		$xml = '<?xml version="1.0" encoding="utf-8"?>' . "\n";
		$xml .= '<rss version="2.0"' . $this->xmlns . '>' . "\n";
	 
		// channel required properties
		$xml .= '<channel>' . "\n";
		$xml .= '<title>' . $this->channel_properties["title"] . '</title>' . "\n";
		$xml .= '<link>' . $this->channel_properties["link"] . '</link>' . "\n";
		$xml .= '<description>' . $this->channel_properties["description"] . '</description>' . "\n";
		$xml .= '<generator>RSS For Airtime v1.0</generator>' . "\n";
		$xml .= '<lastBuildDate>'.date('r').'</lastBuildDate>' . "\n";
		//
		$xml .= '<atom:link href="' . $this->channel_properties["atom_path"]. '" rel="self" type="application/rss+xml" />' . "\n";
		
		// channel optional properties
		$copydate =  date("Y").' - ';
		$xml .= '<copyright>Copyright ' .  $copydate . $this->channel_properties["title"] . '</copyright>' . "\n";
		if(array_key_exists("language", $this->channel_properties)) {
			$xml .= '<language>' . $this->channel_properties["language"] . '</language>' . "\n";
		}
		if(array_key_exists("image_title", $this->channel_properties)) {
			$xml .= '<image>' . "\n";
			$xml .= '<title>' . $this->channel_properties["image_title"] . '</title>' . "\n";
			$xml .= '<link>' . $this->channel_properties["image_link"] . '</link>' . "\n";
			$xml .= '<url>' . $this->channel_properties["image_url"] . '</url>' . "\n";
			$xml .= '</image>' . "\n";
		}
	 
		// get RSS channel items
		$now = date("YmdHis"); // get current time
		$results_items = $this->get_feed_items($now,$this->hours);
		foreach($results_items as $results_item) {
			$xml .= '<item>' . "\n";
			$xml .= '<title>' . $results_item['title'] . '</title>' . "\n";
			$xml .= '<link>' . $results_item['link'] . '</link>' . "\n";
			$xml .= '<description>' . $results_item['description'] . '</description>' . "\n";
			$xml .= '<guid>' . $results_item['guid'] . '</guid>' . "\n";
			$xml .= '<pubDate>' . $results_item['pubDate'] . '</pubDate>' . "\n";
			$xml .= '<category>' . $results_item['category'] . '</category>' . "\n";
			 
			if($this->full_feed) {
				$xml .= '<content:encoded>' . $results_item['content'] . '</content:encoded>' . "\n";
			}
			$xml .= '</item>' . "\n";
		}
		 $xml .= '</channel>';
		 $xml .= '</rss>';
		return $xml;
	}
		
	/*
	* @param $results_date
	* @param $results_items_count
	* @internal param $results_items
	* @return array
	*/
	public function get_feed_items($results_date, $hours) {
		$myshow_ids = array();
		// get rss items according to http://www.rssboard.org/rss-specification
		$rss_feed_items = array();
		$rss_feed_item = array();
		$show_array_items = array();

		$fil = pg_connect($this->db_connect);
		$sql = "SELECT DISTINCT cc_show_instances.id AS inst_id, cc_show_instances.starts AS starts, cc_show_instances.ends AS ends, cc_show_instances.show_id AS show_id, id FROM cc_show_instances WHERE cc_show_instances.starts >= NOW() AND cc_show_instances.starts < NOW() + interval '$hours hours' AND cc_show_instances.modified_instance = 'f' ORDER BY  cc_show_instances.starts ASC";
		$show_instances = pg_query($sql);
		
		$utc = new DateTimeZone('UTC');
	
		// get program start and end date/times
		while($instances = pg_fetch_array($show_instances,NULL,PGSQL_ASSOC)){
			$my_show_inst_id = $instances['inst_id'];
			$my_show_id = $instances['show_id'];
			$my_start = date("D dS M. g:ia", strtotime($instances['starts']));	
			$my_guid = $instances['id'];
			
			if ($this->timezone == 'UTC'){
				$pac_start_time =  new DateTime($instances['starts'], $utc);
				$ny_start_time =  new DateTime($instances['starts'], $utc);
				$uk_start_time =  new DateTime($instances['starts'], $utc);
				
				$uk_start_time = $uk_start_time->setTimezone(new DateTimeZone('Europe/London'));
				$ny_start_time = $ny_start_time->setTimezone(new DateTimeZone('America/New_York'));
				$pac_start_time = $pac_start_time->setTimezone(new DateTimeZone('America/Los_Angeles'));
				
				$my_start_uk = $uk_start_time->format("D dS M. ").'<b>'.$uk_start_time->format("g:ia").'</b>';
				$my_start_ny = $ny_start_time->format("D dS M. ").'<b>'.$ny_start_time->format("g:ia").'</b>';
				$my_start_pac = $pac_start_time->format("D dS M. ").'<b>'.$pac_start_time->format("g:ia").'</b>';	
				
				$pac_end_time =  new DateTime($instances['ends'], $utc);
				$ny_end_time =  new DateTime($instances['ends'], $utc);
				$uk_end_time =  new DateTime($instances['ends'], $utc);
				
				$uk_end_time = $uk_end_time->setTimezone(new DateTimeZone('Europe/London'));
				$ny_end_time = $ny_end_time->setTimezone(new DateTimeZone('America/New_York'));
				$pac_end_time = $pac_end_time->setTimezone(new DateTimeZone('America/Los_Angeles'));	
				
				$my_end_uk = '<b>'.$uk_end_time->format("g:ia").' UK Time</b>';
				$my_end_ny = '<b>'.$ny_end_time->format("g:ia").' Eastern US</b>';
				$my_end_pac = '<b>'. $pac_end_time->format("g:ia").' Pacific US</b>';
				
			} else {
				
				$tz_start_time =  new DateTime($instances['starts'], $utc);
				$tz_start_time = $tz_start_time->setTimezone(new DateTimeZone($this->timezone));
				$my_start_time_tz = $tz_start_time->format("D dS M. ").'<b>'.$tz_start_time->format("g:ia").'</b>';
				
				$tz_end_time =  new DateTime($instances['ends'], $utc);
				$tz_end_time = $tz_end_time->setTimezone(new DateTimeZone($this->timezone));
				$my_end_time_tz = '<b>'.$tz_end_time->format("g:ia").'</b>';
			}
	
			$sql = "SELECT cc_show_instances.ends AS ends FROM cc_show_instances WHERE cc_show_instances.id = ".$my_show_inst_id."  AND cc_show_instances.modified_instance = 'f' ORDER BY cc_show_instances.ends DESC";
			$show_get_end = pg_query($sql);
			$my_end = '';
			
			while($myends = pg_fetch_array($show_get_end,NULL,PGSQL_ASSOC)){
				$my_end = date("g:ia", strtotime($instances['ends']));
				break;
			}
			
			// get show description
			$sql = "SELECT show.description AS show_desc, show.genre AS genre, show.name AS show_name, show.url AS show_url FROM cc_show AS show WHERE show.id = $my_show_id";
			$instances = pg_query($sql);
			while($show_array = pg_fetch_array($instances,NULL,PGSQL_ASSOC)){
				$rss_feed_item['description'] = '';
				$img_url = '';
				
				// title
				$rss_feed_item['title'] = htmlentities($show_array['show_name']);

				// link & icon
				if ( $show_array['show_url'] == '' ) {	
					$rss_feed_item['link'] = $this->site_url;		
				} else {
					// Have we got 2 URLs?
					if (strpos($show_array['show_url'], '|') !== false) { 
						// Seems we have 2 URLS
						$urls = explode("|", $show_array['show_url']);
						// First one is the image - if present.
						if ($urls[0] != '') {
							$img_url = $urls[1];	
						}
						// Second one is link URL
						$rss_feed_item['link'] = $urls[0];
					} else {
						// Use the URL field as it is.
						$rss_feed_item['link'] = $show_array['show_url'];
					}
				}
				
				// combined description
				if($img_url != '') {
					$rss_feed_item['description'] .= htmlentities('<img src="' . $img_url . '" hspace="5" vspace="5" align="left"/>'.$rss_feed_item['title']);
				}
				
				if ($this->timezone == 'UTC'){
					$my_desc = $my_start_uk .'-'.$my_end_uk. '<br />'.$my_start_ny. '-'. $my_end_ny.'<br />'. $my_start_pac .'-'.$my_end_pac.'<br />';
				} else {
					$my_desc = $my_start_time_tz . '-' . $my_end_time_tz . '<br />';
				}
				
				$rss_feed_item['description'] .= htmlentities($my_desc );	
				$rss_feed_item['description'] .= $show_array['show_desc'];
		 
				// pubdate in RFC822 format
				$date = new DateTime();
				$rss_feed_item['pubDate'] = $date->format("r");
				
				// category - using genre for category
				$rss_feed_item['category'] =$show_array['genre'];
		 
				// guid unique identifier
		 		$rss_feed_item['guid'] = $this->site_url.'guid'.$my_guid;
				
				if($this->full_feed) {
					// full content if available - it is not.
					$rss_feed_item['content'] = '<![CDATA[' . $rss_feed_item['description'] . ']]>';
				}	
			} 
			array_push($rss_feed_items, $rss_feed_item);
		} 
		pg_close($fil);
		return $rss_feed_items;
	} // End Function
} // End Class